package samples;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import static java.lang.System.out;

/*
 *
  *   Basic event driven application
 */
public class EventDrivenApp extends Application {


    // -- Handle keyboard -----------------------

    void keyPressed(KeyEvent event) {
        out.println(event.getCode() + " pressed");
    }

    void keyReleased(KeyEvent event) {
        out.println(event.getCode() + " released");
    }

    // ------------- Handle mouse --------------------

    private void mouseOnClicked(MouseEvent mouseEvent) {
        Button b = (Button) mouseEvent.getSource();
        out.println(b.getId() + " clicked");
        out.println(mouseEvent.getX() + " " + mouseEvent.getY());
    }

    private void mouseOnDrag(MouseEvent mouseEvent) {
        out.println(mouseEvent.getX() + " " + mouseEvent.getY());
    }

    // ---- Called before graphics is up -------------------

    @Override
    public void init() {
        out.println("init");
    }


    // --------  JavaFX, setup graphics, event handling and create a timer -----------------------

    final int width = 500;
    final int height = 500;

     // ------- Start up graphics -------------------
    @Override
    public void start(Stage primaryStage) throws Exception {
        VBox root = new VBox();  // The Layout

        Button b1 = new Button("Click me!");
        b1.setOnMouseClicked(this::mouseOnClicked);
        b1.setId("Id 1");
        Button b2 = new Button("Click me too!");
        b2.setOnMouseClicked(this::mouseOnClicked);
        b2.setId("Id 2");

        TextField text = new TextField();
        text.setOnKeyPressed(this::keyPressed);
        text.setOnKeyReleased(this::keyReleased);

        root.getChildren().addAll(b1, b2, text);

        AnimationTimer timer = new AnimationTimer() {
            @Override
            public void handle(long now) {
                if (now % 541 == 0) {
                    out.println("Animation called by timer");
                }
            }
        };
        timer.start();

        Scene scene = new Scene(root);
        scene.setOnMouseDragged(this::mouseOnDrag);
        primaryStage.setScene(scene);
        primaryStage.setTitle("Event demo");
        primaryStage.setWidth(width);
        primaryStage.setHeight(height);
        primaryStage.show();

    }


    public static void main(String[] args) {
        launch(args);
    }
}
