package samples;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static java.lang.System.out;

/*
 *   More subclasses/objects using other other subclasses/objects (object(s) referencing other object(s))
 */
public class MoreClasses {

    public static void main(String[] args) {
        new MoreClasses().program();
    }


    void program() {
        // A list of countries ...
        List<Country> countries = Arrays.asList(new Country("a", 50, 100),
                new Country("b", 100, 50));

        // ... owned by fia
        Player p = new Player("fia", 3, countries);

        // Is fia owner? Yes !
        out.println(p.isOwner(countries.get(0)));

        // Ask fia if she owns country named a
        Country c = p.getCountryByName("a");
        if (c != null) {
            out.println(Double.compare(c.getPopulationDensity(), 2) == 0);
        }


    }

    // ----------- A class for Countries ---------------------------------

    class Country {
        final String name;
        final double area;
        int population;

        Country(String name, double area, int population) {  // Constructor
            this.name = name;
            this.area = area;
            this.population = population;
        }

        double getPopulationDensity() {
            return population / area;
        }

        String getName() {
            return name;
        }

        @Override
        public boolean equals(Object o) {   // Must have
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Country country = (Country) o;

            if (Double.compare(country.area, area) != 0) return false;
            if (population != country.population) return false;
            return name != null ? name.equals(country.name) : country.name == null;

        }

        @Override
        public int hashCode() {  // Must have
            int result;
            long temp;
            result = name != null ? name.hashCode() : 0;
            temp = Double.doubleToLongBits(area);
            result = 31 * result + (int) (temp ^ (temp >>> 32));
            result = 31 * result + population;
            return result;
        }
    }

    // -------------- Class for Player, *using* Country objects ----------------------

    class Player {
        final String name;
        int points;
        final List<Country> countries;

        Player(String name, int points, List<Country> countries) {
            this.name = name;
            this.points = points;
            this.countries = countries;
        }

        boolean isOwner(Country c) {
            return countries.contains(c);
        }

        Country getCountryByName(String name) {
            for (Country c : countries) {
                if (c.getName().equals(name)) {
                    return c;
                }
            }
            return null;
        }
    }
}




