package samples.oo.classes;

import java.util.ArrayList;
import java.util.List;

import static java.lang.System.out;

/*
        Implementing own equals, toString and hashCode
        (inherited from Object but mostly need own versions)

        Equals needed to compare object, used by collections like List

 */
public class EqualsHashToString {

    public static void main(String[] args) {
        new EqualsHashToString().program();
    }

    void program() {
        Player p1 = new Player("Lisa", 4);
        out.println(p1);  // Will call toString implicitly

        Player p2 = new Player("Olle", 2);
        Player p3 = new Player("Olle", 6);
        out.println(p2 != p3);
        out.println(p2.equals(p3));
        out.println(!p2.equals(p1));

        List<Player> ps = new ArrayList<>();
        ps.add(p2);
        out.println(ps.contains(p3)); // Comment out equals and test

    }

    class Player /* extends (inherits) Object */ {   // Implicit inherits from standard class Object
        final String name;
        int points;

        Player(String name, int points) {
            this.name = name;
            this.points = points;
        }

        String getName() {
            return name;      // Same as this.name
        }

        @Override                         // @Override to make compiler check method head is same as in Object
        public String toString() {               // Own version of inherited method from Object
            return "Player{" +
                    "name='" + name + '\'' +   // More human friendly output, called by out.println()
                    ", points=" + points +
                    '}';
        }

        @Override
        public boolean equals(Object o) {           // Own version of inherited method from Object
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }
            Player player = (Player) o;
            return name.equals(player.name);
        }

        @Override
        public int hashCode() {
            return name != null ? name.hashCode() : 0;
        }  // Own version of inheritec method from Object
    }

}
