package pong.model;

public abstract class AbstractPongObject implements IDrawable {

    private final double width;
    private final double height;
    private double x;     // Upper left corner in enclosing square
    private double y;

    public AbstractPongObject(double x, double y, double width, double height) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }

    public boolean intersects(AbstractPongObject other) {
        if (other == null) {
            return false;
        }
        boolean above = other.getMaxY() < getMinY();
        boolean below = other.getMinY() > getMaxY();
        boolean leftOf = other.getMaxX() < getMinX();
        boolean rightOf = other.getMinX() > getMaxX();
        return !(above || below || leftOf || rightOf);
    }


    public void setX(double x) {
        this.x = x;
    }

    public void setY(double y) {
        this.y = y;
    }

    @Override
    public double getMinX() {
        return x;
    }

    @Override
    public double getMinY() {
        return y;
    }

    @Override
    public double getWidth() {
        return width;
    }

    @Override
    public double getHeight() {
        return height;
    }

    @Override
    public String discriminator() {
        return "";
    }  // Default implementation

    public double getMaxX() {
        return x + width;
    }

    public double getMaxY() {
        return y + height;
    }

}
